#! /usr/bin/python
# PyMail DTN Nomadic Mail System
# Copyright (C) Folly Consulting Ltd, 2009, 2010
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#


"""
Postfix pipe client designed to squirrel mail messages in a temporary maildir
and send them out as dtn bundles.

Intended to take messages for a specific DTN association (at present).

The messages are passed on across a TCP connection to the main handler.

The reason for this small complexity is that it makes it simpler to interface
with Postfix and use the capabilities of the pipe daemon.  This has lots of
options that can be usefully used here, such as one mail message per invocation
and adding Delivered To headers and simulating bsmtp capabilities.

In the event of problems we can return error statuses from sysexits to tell
Postfix what happened.

Revision History
================
Version   Date       Author         Notes
0.1       14/03/2010 Elwyn Davies   License statement added.
0.0	  25/07/2009 Elwyn Davies   Created for N4C Summer tests 2009
"""

import sys
import socket
import logging
import logging.config
import logging.handlers

# Selected values from sysexits.h
# Postfix pipe command understands these and will do the right thing allegedly.
EX_OK = 0
EX_OSERR = 71
EX_IOERR = 74
EX_TEMPFAIL = 75

# Setup logging
logging.config.fileConfig("/home/dtn/dtn/dtn_pymail_log.conf")
pymail_logger = logging.getLogger("dtn_postfix_pipe")
loginfo = pymail_logger.info
logdebug = pymail_logger.debug
logerror = pymail_logger.error

loginfo("DTN mail Postfix transport shim running")

# Open a socket to the outgoing mail handler
HOST, SERVER_PORT, ANY_PORT = "localhost", 2111, 0
try:
    skt = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
except socket.error, msg:
    logerror("Socket creation failed")
    sys.exit(EX_OSERR)

try:
    skt.bind((HOST, ANY_PORT,))
    skt.connect((HOST, SERVER_PORT))
except socket.error, msg:
    logerror("Transport shim: Couldn't connect")
    skt.close()
    # Indicate we should try again later if can't connect to server process
    sys.exit(EX_TEMPFAIL)


# Copy the message to the server
try:
    while (True):
        next_line = sys.stdin.readline()
        logdebug(next_line)
        if next_line == "":
            break
        skt.send(next_line)
except:
    skt.close()
    sys.exit(EX_IOERR)
loginfo("Transport shim: finished")
skt.close()
sys.exit(EX_OK)


